from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.commands.acls import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkAcls(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.kafka_cluster = "my-cluster"
        self.namespace = "kafka"
        self.topic = "my-topic"
        self.group = "my-group"
        self.user = "my-user"

    @mock.patch("kfk.commands.acls.os")
    def test_list_all_acls(self, mock_os):
        result = self.runner.invoke(
            kfk, ["acls", "--list", "-n", self.namespace, "-c", self.kafka_cluster]
        )
        assert result.exit_code == 0
        native_command = (
            "bin/kafka-acls.sh --authorizer-properties"
            " zookeeper.connect=localhost:12181 --list  "
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{kafka_cluster}-zookeeper-0")
            .container("zookeeper")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(kafka_cluster=self.kafka_cluster)
        )

    @mock.patch("kfk.commands.acls.os")
    def test_list_topic_acls(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "acls",
                "--list",
                "--topic",
                self.topic,
                "-n",
                self.namespace,
                "-c",
                self.kafka_cluster,
            ],
        )
        assert result.exit_code == 0
        native_command = (
            "bin/kafka-acls.sh --authorizer-properties"
            " zookeeper.connect=localhost:12181 --list --topic {topic} ".format(
                topic=self.topic
            )
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{kafka_cluster}-zookeeper-0")
            .container("zookeeper")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(kafka_cluster=self.kafka_cluster)
        )

    @mock.patch("kfk.commands.acls.os")
    def test_list_group_acls(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "acls",
                "--list",
                "--group",
                self.group,
                "-n",
                self.namespace,
                "-c",
                self.kafka_cluster,
            ],
        )
        assert result.exit_code == 0
        native_command = (
            "bin/kafka-acls.sh --authorizer-properties"
            " zookeeper.connect=localhost:12181 --list  --group {group}".format(
                group=self.group
            )
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{kafka_cluster}-zookeeper-0")
            .container("zookeeper")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(kafka_cluster=self.kafka_cluster)
        )

    @mock.patch("kfk.commands.acls.os")
    def test_add_acls_without_principal(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "acls",
                "--add",
                "--topic",
                self.topic,
                "-n",
                self.namespace,
                "-c",
                self.kafka_cluster,
            ],
        )
        assert result.exit_code == 2

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_add_topic_acl(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "acls",
                    "--add",
                    "--topic",
                    self.topic,
                    "--allow-principal",
                    "User:" + self.user,
                    "--operation",
                    "Read",
                    "-n",
                    self.namespace,
                    "-c",
                    self.kafka_cluster,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_with_one_topic_acl.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_remove_topic_acl(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/user_with_authorization_with_one_topic_acl.yaml"
        ) as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "acls",
                    "--remove",
                    "--topic",
                    self.topic,
                    "--allow-principal",
                    "User:" + self.user,
                    "--operation",
                    "Read",
                    "-n",
                    self.namespace,
                    "-c",
                    self.kafka_cluster,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_add_topic_acl_to_one_acl(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/user_with_authorization_with_one_topic_acl.yaml"
        ) as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "acls",
                    "--add",
                    "--topic",
                    self.topic,
                    "--allow-principal",
                    "User:" + self.user,
                    "--operation",
                    "Write",
                    "-n",
                    self.namespace,
                    "-c",
                    self.kafka_cluster,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_with_two_topic_acls.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_remove_topic_acl_from_two_acls(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/user_with_authorization_with_two_topic_acls.yaml"
        ) as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "acls",
                    "--remove",
                    "--topic",
                    self.topic,
                    "--allow-principal",
                    "User:" + self.user,
                    "--operation",
                    "Write",
                    "-n",
                    self.namespace,
                    "-c",
                    self.kafka_cluster,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_with_one_topic_acl.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_add_topic_acl_with_two_operations(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/user_with_authorization_with_one_topic_acl.yaml"
        ) as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "acls",
                    "--add",
                    "--topic",
                    self.topic,
                    "--allow-principal",
                    "User:" + self.user,
                    "--operation",
                    "Write",
                    "--operation",
                    "Describe",
                    "-n",
                    self.namespace,
                    "-c",
                    self.kafka_cluster,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_with_three_topic_acls.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_add_group_acl(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/user_with_authorization_with_one_topic_acl.yaml"
        ) as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "acls",
                    "--add",
                    "--group",
                    self.group,
                    "--allow-principal",
                    "User:" + self.user,
                    "--operation",
                    "Read",
                    "-n",
                    self.namespace,
                    "-c",
                    self.kafka_cluster,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_"
                "with_one_topic_and_one_group_acls.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml



import re


def convert_string_to_type(str_val):
    str_val = str(str_val)
    if str_val.isdigit():
        return int(str_val)
    elif is_number(str_val):
        return float(str_val)
    elif is_bool(str_val):
        return convert_string_to_boolean(str_val)
    else:
        return str_val


def is_number(str_val):
    try:
        float(str_val)
    except ValueError:
        return False
    return True


def is_bool(str_val):
    if str_val == "true" or str_val == "false":
        return True
    else:
        return False


def convert_string_to_boolean(str_val):
    if str_val == "true":
        return True
    else:
        return False


def snake_to_camel_case(snake_str):
    components = snake_str.split("_")
    return components[0] + "".join(x.title() for x in components[1:])


def get_list_by_split_string(str_val, split_char):
    # TODO: exception here
    return str_val.split(split_char)


def is_valid_url(url):
    regex = re.compile(
        r"^(?:http|ftp)s?://"
        r"(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+"
        r"(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|localhost|"
        r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"
        r"(?::\d+)?"
        r"(?:/?|[/?]\S+)$",
        re.IGNORECASE,
    )

    return re.match(regex, url) is not None



from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.commands.connect.connectors import connect
from kfk.kubectl_command_builder import Kubectl


class TestKfkConnectors(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-connect-cluster"
        self.namespace = "kafka"
        self.connector_config_file = "tests/files/twitter-connector.properties"
        self.connector = "twitter-source-connector"

    def test_no_option(self):
        result = self.runner.invoke(
            connect,
            [
                "connectors",
                self.connector_config_file,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 1
        assert "Missing options: kfk connectors" in result.output

    @mock.patch("kfk.commands.connect.connectors.os")
    def test_list_connectors(self, mock_os):
        result = self.runner.invoke(
            connect, ["connectors", "--list", "-c", self.cluster, "-n", self.namespace]
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .get()
            .kafkaconnectors()
            .label("strimzi.io/cluster={cluster}")
            .namespace(self.namespace)
            .build()
            .format(cluster=self.cluster)
        )

    @mock.patch("kfk.commands.connect.connectors.os")
    def test_describe_connector(self, mock_os):
        result = self.runner.invoke(
            connect,
            [
                "connectors",
                "--describe",
                "--connector",
                self.connector,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .describe()
            .kafkaconnectors(self.connector)
            .namespace(self.namespace)
            .build()
        )

    @mock.patch("kfk.commands.connect.connectors.os")
    def test_describe_connector_output_yaml(self, mock_os):
        result = self.runner.invoke(
            connect,
            [
                "connectors",
                "--describe",
                "--connector",
                self.connector,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
                "-o",
                "yaml",
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .get()
            .kafkaconnectors(self.connector)
            .namespace(self.namespace)
            .output("yaml")
            .build()
        )

    @mock.patch("kfk.commands.connect.connectors.delete_using_yaml")
    def test_delete_connector(self, mock_delete_using_yaml):
        result = self.runner.invoke(
            connect,
            [
                "connectors",
                "--delete",
                "--connector",
                self.connector,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )

        assert result.exit_code == 0

        mock_delete_using_yaml.assert_called_once()

    def test_create_connector_without_config_file(self):
        result = self.runner.invoke(
            connect,
            ["connectors", "--create", "-c", self.cluster, "-n", self.namespace],
        )

        assert result.exit_code == 2

    @mock.patch("kfk.commands.connect.connectors.create_temp_file")
    @mock.patch("kfk.commands.connect.connectors.create_using_yaml")
    def test_create_connector(self, mock_create_using_yaml, mock_create_temp_file):
        result = self.runner.invoke(
            connect,
            [
                "connectors",
                "--create",
                self.connector_config_file,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )

        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-connect-connector-twitter.yaml") as file:
            expected_connector_yaml = file.read()
            result_connector_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connector_yaml == result_connector_yaml

        mock_create_using_yaml.assert_called_once()

    def test_alter_connector_without_config_file(self):
        result = self.runner.invoke(
            connect, ["connectors", "--alter", "-c", self.cluster, "-n", self.namespace]
        )

        assert result.exit_code == 2

    @mock.patch("kfk.commands.connect.connectors.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.connect.connectors.os")
    def test_alter_connector(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-connect-connector-twitter.yaml") as file:
            connector_yaml = file.read()
            mock_get_resource_yaml.return_value = connector_yaml

            result = self.runner.invoke(
                connect,
                [
                    "connectors",
                    "--alter",
                    "tests/files/twitter_connector_with_config_change.properties",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )

            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-connect-connector-"
                "twitter_with_config_change.yaml"
            ) as file:
                expected_connector_yaml = file.read()
                result_connector_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_connector_yaml == result_connector_yaml



SPACE = " "
COLON = ":"
SEMICOLON = ";"
AMPERSAND = "&"
COMMA = ","
EQUALS = "="
NEW_LINE = "\n"
KAFKA_PORT = "9092"
KAFKA_SECURE_PORT = "9093"
KAFKA_SSL = "SSL"
COMMON_NAME_PREFIX = "CN="
BROKER_TMP_FOLDER_PATH = "/tmp"
CONNECT_OUTPUT_TYPE_DOCKER = "docker"
EXTENSION_TAR_GZ = ".tar.gz"
EXTENSION_JAR = ".jar"
EXTENSION_ZIP = ".zip"
TRUE = "true"


class SpecialTexts:
    BROKER_CONFIG_FILE_USER_CONFIG_HEADER = (
        "\n##########\n# User provided configuration\n##########\n"
    )
    CONNECT_BOOTSTRAP_SERVERS = "bootstrap.servers"
    CONNECT_IMAGE = "image"
    CONNECT_PLUGIN_URL = "plugin.url"
    CONNECT_PLUGIN_PATH = "plugin.path"
    CONNECTOR_NAME = "name"
    CONNECTOR_TASKS_MAX = "tasks.max"
    CONNECTOR_CLASS = "connector.class"
    OPERATOR_ROLE_BINDING = "RoleBinding"
    OPERATOR_MY_PROJECT = "myproject"



import os
import ssl
import stat
import subprocess
import tarfile
from pathlib import Path

import wget

from kfk.config import (
    BASE_PATH,
    KUBECTL_PATH,
    KUBECTL_RELEASE_URL,
    KUBECTL_VERSION,
    STRIMZI_PATH,
    STRIMZI_RELEASE_URL,
    STRIMZI_VERSION,
)
from kfk.kubectl_command_builder import Kubectl

ssl._create_default_https_context = ssl._create_unverified_context


def setup():
    download_kubectl_if_not_exists()
    update_kubectl_if_new_version_exists()
    download_strimzi_if_not_exists()


def download_kubectl_if_not_exists():
    if not os.path.exists(KUBECTL_PATH):
        print(f"Creating Strimzi Kafka CLI Dependencies folder: {BASE_PATH}\n")
        Path(BASE_PATH).mkdir(exist_ok=True)

        _download_kubectl()


def update_kubectl_if_new_version_exists():
    if (
        os.path.exists(KUBECTL_PATH)
        and os.environ.get("STRIMZI_KAFKA_CLI_KUBECTL_VERSION") is None
        and os.environ.get("STRIMZI_KAFKA_CLI_KUBECTL_PATH") is None
        and KUBECTL_VERSION
        not in subprocess.check_output(
            Kubectl().version("--client=true").build(),
            shell=True,
            stderr=subprocess.STDOUT,
        ).decode("utf-8")
    ):
        os.rename(KUBECTL_PATH, KUBECTL_PATH + "_old")
        _download_kubectl()


def _download_kubectl():
    print(f"Downloading dependency: kubectl {KUBECTL_VERSION}...\n")
    wget.download(KUBECTL_RELEASE_URL, KUBECTL_PATH)
    print("\nDownload successfully completed!\n")
    current_stat = os.stat(KUBECTL_PATH)
    os.chmod(KUBECTL_PATH, current_stat.st_mode | stat.S_IEXEC)


def download_strimzi_if_not_exists():
    if not os.path.exists(STRIMZI_PATH):
        strimzi_tarfile_path = STRIMZI_PATH + ".tar.gz"

        print(f"Creating Strimzi Kafka CLI Dependencies folder: {BASE_PATH}\n")
        Path(BASE_PATH).mkdir(exist_ok=True)

        print(f"Downloading dependency: Strimzi {STRIMZI_VERSION}...\n")
        wget.download(STRIMZI_RELEASE_URL, strimzi_tarfile_path)
        print("\nDownload successfully completed!\n")
        print(f"Extracting Strimzi {STRIMZI_VERSION}...\n")
        tar = tarfile.open(strimzi_tarfile_path)
        tar.extractall(path=BASE_PATH)
        tar.close()
        os.remove(strimzi_tarfile_path)



from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.commands.configs import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkConfigs(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.topic = "my-topic"
        self.user = "my-user"
        self.broker_count = 3

    def test_no_option(self):
        result = self.runner.invoke(
            kfk,
            [
                "configs",
                "--entity-type",
                "topics",
                "--entity-name",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 1
        assert "Missing options: kfk configs" in result.output

    def test_wrong_entity_type(self):
        result = self.runner.invoke(
            kfk,
            [
                "configs",
                "--entity-type",
                "foos",
                "--entity-name",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 2

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_add_one_topic_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic_without_config.yaml") as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--alter",
                    "--add-config",
                    "min.insync.replicas=3",
                    "--entity-type",
                    "topics",
                    "--entity-name",
                    self.topic,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/topic_with_one_config.yaml") as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_add_two_topic_configs(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic_without_config.yaml") as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--alter",
                    "--add-config",
                    "min.insync.replicas=3,cleanup.policy=compact",
                    "--entity-type",
                    "topics",
                    "--entity-name",
                    self.topic,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/topic_with_two_configs.yaml") as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_delete_one_topic_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic_with_two_configs.yaml") as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--alter",
                    "--delete-config",
                    "cleanup.policy",
                    "--entity-type",
                    "topics",
                    "--entity-name",
                    self.topic,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/topic_with_one_config.yaml") as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.os")
    def test_describe_topic_config(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "configs",
                "--describe",
                "--entity-type",
                "topics",
                "--entity-name",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .describe()
            .kafkatopics(self.topic)
            .namespace(self.namespace)
            .build()
        )

    @mock.patch("kfk.commands.configs.os")
    def test_describe_topic_config_native(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "configs",
                "--describe",
                "--entity-type",
                "topics",
                "--entity-name",
                self.topic,
                "--native",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        native_command = (
            "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:9092"
            " --entity-type topics --describe --entity-name {entity_name}"
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(cluster=self.cluster, entity_name=self.topic)
        )

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_add_one_user_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_scram.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--alter",
                    "--add-config",
                    "request_percentage=55",
                    "--entity-type",
                    "users",
                    "--entity-name",
                    self.user,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_one_quota.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_add_two_user_configs(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_scram.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--alter",
                    "--add-config",
                    "request_percentage=55,consumer_byte_rate=2097152",
                    "--entity-type",
                    "users",
                    "--entity-name",
                    self.user,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_two_quotas.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_delete_one_user_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_two_quotas.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--alter",
                    "--delete-config",
                    "consumer_byte_rate",
                    "--entity-type",
                    "users",
                    "--entity-name",
                    self.user,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_one_quota.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.os")
    def test_describe_user_config(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "configs",
                "--describe",
                "--entity-type",
                "users",
                "--entity-name",
                self.user,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkausers(self.user).namespace(self.namespace).build()
        )

    @mock.patch("kfk.commands.configs.os")
    def test_describe_user_config_native(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "configs",
                "--describe",
                "--entity-type",
                "users",
                "--entity-name",
                self.user,
                "--native",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        native_command = (
            "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:9092"
            " --entity-type users --describe --entity-name CN={entity_name}"
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(cluster=self.cluster, entity_name=self.user)
        )

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_add_one_broker_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-ephemeral.yaml") as file:
            cluster_yaml = file.read()
            mock_get_resource_yaml.return_value = cluster_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--alter",
                    "--add-config",
                    "unclean.leader.election.enable=true",
                    "--entity-type",
                    "brokers",
                    "--entity-name",
                    self.cluster,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-ephemeral_with_one_additional_config.yaml"
            ) as file:
                expected_cluster_yaml = file.read()
                result_cluster_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_cluster_yaml == result_cluster_yaml

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_delete_one_broker_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/kafka-ephemeral_with_one_additional_config.yaml"
        ) as file:
            cluster_yaml = file.read()
            mock_get_resource_yaml.return_value = cluster_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--alter",
                    "--delete-config",
                    "unclean.leader.election.enable",
                    "--entity-type",
                    "brokers",
                    "--entity-name",
                    "all",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-ephemeral_two_additional_configs_deleted.yaml"
            ) as file:
                expected_cluster_yaml = file.read()
                result_cluster_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_cluster_yaml == result_cluster_yaml

    @mock.patch("kfk.commands.clusters.os")
    def test_describe_broker_config(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "configs",
                "--describe",
                "--entity-type",
                "brokers",
                "--entity-name",
                "all",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkas(self.cluster).namespace(self.namespace).build()
        )

    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.configs.os")
    def test_describe_broker_config_native(
        self, mock_os, mock_get_config_resource_yaml
    ):
        with open("tests/files/yaml/kafka-config.yaml") as file:
            config_yaml = file.read()
            mock_get_config_resource_yaml.return_value = config_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--describe",
                    "--entity-type",
                    "brokers",
                    "--native",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0
            assert (
                "All user provided configs for brokers in the cluster are:"
                in result.output
            )
            assert "log.message.format.version=2.6" in result.output
            assert "offsets.topic.replication.factor=3" in result.output
            assert "transaction.state.log.min.isr=2" in result.output
            assert "transaction.state.log.replication.factor=3" in result.output

            native_command = (
                "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:9092"
                " --entity-type brokers --describe"
            )

            mock_os.system.assert_called_with(
                Kubectl()
                .exec("-it", "{cluster}-kafka-0")
                .container("kafka")
                .namespace(self.namespace)
                .exec_command(native_command)
                .build()
                .format(cluster=self.cluster, entity_name=self.broker_count - 1)
            )

    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.configs.os")
    def test_describe_broker_config_native_with_id(
        self, mock_os, mock_get_config_resource_yaml
    ):
        with open("tests/files/yaml/kafka-config.yaml") as file:
            config_yaml = file.read()
            mock_get_config_resource_yaml.return_value = config_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "configs",
                    "--describe",
                    "--entity-type",
                    "brokers",
                    "--entity-name",
                    "0",
                    "--native",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0
            assert (
                "All user provided configs for brokers in the cluster are:"
                in result.output
            )
            assert "log.message.format.version=2.6" in result.output
            assert "offsets.topic.replication.factor=3" in result.output
            assert "transaction.state.log.min.isr=2" in result.output
            assert "transaction.state.log.replication.factor=3" in result.output

            native_command = (
                "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:9092"
                " --entity-type brokers --describe --entity-name 0"
            )

            mock_os.system.assert_called_with(
                Kubectl()
                .exec("-it", "{cluster}-kafka-0")
                .container("kafka")
                .namespace(self.namespace)
                .exec_command(native_command)
                .build()
                .format(cluster=self.cluster, entity_name=self.broker_count - 1)
            )



import os
import platform
import sys
from pathlib import Path

STRIMZI_VERSION = "0.37.0"
KUBECTL_VERSION = "v1.28.3"


def _get_processor_type():
    if _is_64_bit():
        if "arm" in platform.uname().machine or "aarch" in platform.uname().machine:
            return "arm64"
        else:
            return "amd64"
    else:
        if "arm" in platform.uname().machine:
            return "arm"
        else:
            return "386"


def _is_64_bit():
    return sys.maxsize > 2**32


BASE_FOLDER = ".strimzi-kafka-cli"
BASE_PATH = (
    (str(Path.home()) + "/" + BASE_FOLDER)
    if os.environ.get("STRIMZI_KAFKA_CLI_BASE_PATH") is None
    else os.environ.get("STRIMZI_KAFKA_CLI_BASE_PATH")
)
STRIMZI_VERSION = (
    STRIMZI_VERSION
    if os.environ.get("STRIMZI_KAFKA_CLI_STRIMZI_VERSION") is None
    else os.environ.get("STRIMZI_KAFKA_CLI_STRIMZI_VERSION")
)
STRIMZI_PATH = (
    (BASE_PATH + f"/strimzi-{STRIMZI_VERSION}")
    if os.environ.get("STRIMZI_KAFKA_CLI_STRIMZI_PATH") is None
    else os.environ.get("STRIMZI_KAFKA_CLI_STRIMZI_PATH")
)
STRIMZI_RELEASE_URL = (
    f"https://github.com/strimzi/strimzi-kafka-operator/releases/"
    f"download/{STRIMZI_VERSION}/strimzi-{STRIMZI_VERSION}.tar.gz"
)
KUBECTL_VERSION = (
    KUBECTL_VERSION
    if os.environ.get("STRIMZI_KAFKA_CLI_KUBECTL_VERSION") is None
    else os.environ.get("STRIMZI_KAFKA_CLI_KUBECTL_VERSION")
)
KUBECTL = "kubectl" if platform.system().lower() != "windows" else "kubectl.exe"
KUBECTL_PATH = (
    (BASE_PATH + "/" + KUBECTL)
    if os.environ.get("STRIMZI_KAFKA_CLI_KUBECTL_PATH") is None
    else os.environ.get("STRIMZI_KAFKA_CLI_KUBECTL_PATH")
)
PROCESSOR_TYPE = _get_processor_type()
KUBECTL_RELEASE_URL = (
    f"https://storage.googleapis.com/kubernetes-release/release/"
    f"{KUBECTL_VERSION}/bin/{platform.system().lower()}/"
    f"{PROCESSOR_TYPE}/{KUBECTL}"
)



from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.commands.clusters import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkClusters(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.new_cluster_name = "my-cluster-with-new-name"

    def test_no_option(self):
        result = self.runner.invoke(
            kfk, ["clusters", "--cluster", self.cluster, "-n", self.namespace]
        )
        assert result.exit_code == 1
        assert "Missing options: kfk clusters" in result.output

    @mock.patch("kfk.commands.clusters.os")
    def test_list_clusters(self, mock_os):
        result = self.runner.invoke(kfk, ["clusters", "--list", "-n", self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkas().namespace(self.namespace).build()
        )

    @mock.patch("kfk.commands.clusters.os")
    def test_list_clusters_all_namespaces(self, mock_os):
        result = self.runner.invoke(kfk, ["clusters", "--list"])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().get().kafkas().namespace().build())

    @mock.patch("kfk.commands.clusters.os")
    def test_describe_cluster(self, mock_os):
        result = self.runner.invoke(
            kfk,
            ["clusters", "--describe", "--cluster", self.cluster, "-n", self.namespace],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkas(self.cluster).namespace(self.namespace).build()
        )

    @mock.patch("kfk.commands.clusters.os")
    def test_describe_cluster_output_yaml(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "clusters",
                "--describe",
                "--cluster",
                self.cluster,
                "-n",
                self.namespace,
                "-o",
                "yaml",
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .get()
            .kafkas(self.cluster)
            .namespace(self.namespace)
            .output("yaml")
            .build()
        )

    @mock.patch("kfk.commands.clusters.os")
    def test_alter_cluster_without_parameters(self, mock_os):
        result = self.runner.invoke(
            kfk,
            ["clusters", "--alter", "--cluster", self.cluster, "-n", self.namespace],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().edit().kafkas(self.cluster).namespace(self.namespace).build()
        )

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_alter_cluster_with_one_additional_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-ephemeral.yaml") as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "clusters",
                    "--alter",
                    "--config",
                    "unclean.leader.election.enable=true",
                    "--cluster",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-ephemeral_with_one_additional_config.yaml"
            ) as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_alter_cluster_with_two_additional_configs(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-ephemeral.yaml") as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "clusters",
                    "--alter",
                    "--config",
                    "unclean.leader.election.enable=true",
                    "--config",
                    "log.retention.hours=168",
                    "--cluster",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-ephemeral_with_two_additional_configs.yaml"
            ) as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_alter_cluster_with_two_additional_configs_delete_one_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/kafka-ephemeral_with_two_additional_configs.yaml"
        ) as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "clusters",
                    "--alter",
                    "--delete-config",
                    "log.retention.hours",
                    "--cluster",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-ephemeral_with_one_additional_config.yaml"
            ) as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_alter_cluster_with_two_additional_configs_delete_two_configs(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/kafka-ephemeral_with_two_additional_configs.yaml"
        ) as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "clusters",
                    "--alter",
                    "--delete-config",
                    "log.retention.hours",
                    "--delete-config",
                    "unclean.leader.election.enable",
                    "--cluster",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-ephemeral_two_additional_configs_deleted.yaml"
            ) as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_alter_cluster_with_one_replica(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-ephemeral_name_updated.yaml") as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "clusters",
                    "--alter",
                    "--replicas",
                    1,
                    "--cluster",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/kafka-ephemeral_with_one_replica.yaml") as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_alter_cluster_with_two_replicas(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-ephemeral_name_updated.yaml") as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "clusters",
                    "--alter",
                    "--replicas",
                    2,
                    "--cluster",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-ephemeral_with_two_replicas.yaml"
            ) as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.clusters.os")
    def test_alter_cluster_with_one_replica_one_zk_replica(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-ephemeral_name_updated.yaml") as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(
                kfk,
                [
                    "clusters",
                    "--alter",
                    "--replicas",
                    1,
                    "--zk-replicas",
                    1,
                    "--cluster",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-ephemeral_with_one_replica_one_zk_replica.yaml"
            ) as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch("kfk.commands.clusters.click.confirm")
    @mock.patch("kfk.commands.clusters.delete_using_yaml")
    def test_delete_cluster(self, mock_delete_using_yaml, mock_click_confirm):
        mock_click_confirm.return_value = True
        result = self.runner.invoke(
            kfk,
            ["clusters", "--delete", "--cluster", self.cluster, "-n", self.namespace],
        )
        assert result.exit_code == 0
        mock_delete_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.clusters.click.confirm")
    @mock.patch("kfk.commands.clusters.delete_using_yaml")
    def test_delete_cluster_with_yes_flag(
        self, mock_delete_using_yaml, mock_click_confirm
    ):
        mock_click_confirm.return_value = False
        result = self.runner.invoke(
            kfk,
            [
                "clusters",
                "--delete",
                "--cluster",
                self.cluster,
                "-n",
                self.namespace,
                "-y",
            ],
        )
        assert result.exit_code == 0

        mock_delete_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commands.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.clusters.click.confirm")
    @mock.patch("kfk.commands.clusters.create_using_yaml")
    def test_create_cluster(
        self,
        mock_create_using_yaml,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
    ):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(
            kfk,
            [
                "clusters",
                "--create",
                "--cluster",
                self.new_cluster_name,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-ephemeral_name_updated.yaml") as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commands.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.clusters.click.confirm")
    @mock.patch("kfk.commands.clusters.create_using_yaml")
    def test_create_cluster_with_yes_flag(
        self,
        mock_create_using_yaml,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
    ):
        mock_click_confirm.return_value = False

        result = self.runner.invoke(
            kfk,
            [
                "clusters",
                "--create",
                "--cluster",
                self.new_cluster_name,
                "-n",
                self.namespace,
                "-y",
            ],
        )
        assert result.exit_code == 0

        mock_open_file_in_system_editor.assert_not_called()
        with open("tests/files/yaml/kafka-ephemeral_name_updated.yaml") as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commands.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.clusters.click.confirm")
    @mock.patch("kfk.commands.clusters.create_using_yaml")
    def test_create_cluster_with_one_replica(
        self,
        mock_create_using_yaml,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
    ):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(
            kfk,
            [
                "clusters",
                "--create",
                "--cluster",
                self.new_cluster_name,
                "--replicas",
                1,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-ephemeral_with_one_replica.yaml") as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commands.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.clusters.click.confirm")
    @mock.patch("kfk.commands.clusters.create_using_yaml")
    def test_create_cluster_with_two_replicas(
        self,
        mock_create_using_yaml,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
    ):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(
            kfk,
            [
                "clusters",
                "--create",
                "--cluster",
                self.new_cluster_name,
                "--replicas",
                2,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-ephemeral_with_two_replicas.yaml") as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commands.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.clusters.click.confirm")
    @mock.patch("kfk.commands.clusters.create_using_yaml")
    def test_create_cluster_with_three_replicas(
        self,
        mock_create_using_yaml,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
    ):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(
            kfk,
            [
                "clusters",
                "--create",
                "--cluster",
                self.new_cluster_name,
                "--replicas",
                3,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-ephemeral_name_updated.yaml") as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.clusters.create_temp_file")
    @mock.patch("kfk.commands.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.clusters.click.confirm")
    @mock.patch("kfk.commands.clusters.create_using_yaml")
    def test_create_cluster_with_one_replica_one_zk_replica(
        self,
        mock_create_using_yaml,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
    ):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(
            kfk,
            [
                "clusters",
                "--create",
                "--cluster",
                self.new_cluster_name,
                "--replicas",
                1,
                "--zk-replicas",
                1,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        with open(
            "tests/files/yaml/kafka-ephemeral_with_one_replica_one_zk_replica.yaml"
        ) as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

        mock_create_using_yaml.assert_called_once()
